/*------------------------------------------------
The next series of functions were originally saved
in the CMDPROC.C file.  These functions process
a single command.
------------------------------------------------*/

#include <ctype.h>
#include <string.h>
#include "tdp.h"

/*----------------------------------------------------------------------  
--------
------------------------------------------------------------------------  
------*/
static pdata char helptext [] =
  "\r\n\r\n"
  "HELP:\r\n"
  "STCLK hhmmss -- Set Clock Time\r\n"
  "RDCLK        -- Display Clock Time\r\n"
  "SCCLK ON|OFF -- Display Clock Time Every Second\r\n"
  "STALM hhmm   -- Set Alarm\r\n"
  "RDALM        -- Display Alarm Time\r\n"
  "CLALM        -- Clear Alarm Time\r\n"
  "\r\n";

/*----------------------------------------------------------------------  
--------
Commands:
HELP
STCLK HHMMSS
RDCLK
SCCLK
STALM HHMM
RDALM
CLALM

Responses:
TIME: HH:MM:SS.HH	(current time)
ALARM: HH:MM		(alarm time)
<beep>			(alarm - for one minute only)
------------------------------------------------------------------------  
------*/
enum
  {
  CID_SET_CLK,
  CID_READ_CLK,
  CID_SCAN_CLK,

  CID_SET_ALM,
  CID_READ_ALM,
  CID_CLR_ALM,

  CID_LAST
  };

struct cmd_st
  {
  const char *cmdstr;
  unsigned char id;
  };

static pdata struct cmd_st cmd_tbl [] =
  {
    { "STCLK",		CID_SET_CLK },
    { "RDCLK",		CID_READ_CLK },
    { "SCCLK",		CID_SCAN_CLK },
    { "STALM",		CID_SET_ALM },
    { "RDALM",		CID_READ_ALM },
    { "CLALM",		CID_CLR_ALM },
  };

#define CMD_TBL_LEN (sizeof (cmd_tbl) / sizeof (cmd_tbl [0]))

/*----------------------------------------------------------------------  
--------
This function searches the cmd_tbl for a specific command and returns the 
ID
associated with that command or CID_LAST if there is no matching command.
------------------------------------------------------------------------  
------*/
static unsigned char cmdid_search (
  char *cmdstr)
{
struct cmd_st *ctp;

for (ctp = cmd_tbl; ctp < &cmd_tbl [CMD_TBL_LEN]; ctp++)
  {
  if (strcmp (ctp->cmdstr, cmdstr) == 0)
    return (ctp->id);
  }

return (CID_LAST);
}

/*----------------------------------------------------------------------  
--------
No programmer's library would be complete without strupr!
------------------------------------------------------------------------  
------*/
char *strupr (
  char *src)
{
char *s;

for (s = src; *s != '\0'; s++)
  *s = toupper (*s);

return (src);
}

/*----------------------------------------------------------------------  
--------
This function processes the cmd command.
------------------------------------------------------------------------  
------*/
void cmd_proc (
  const char *cmd)
{
xdata char cmdstr_buf [1 + MAX_CMD_LEN];
xdata char argstr_buf [1 + MAX_CMD_LEN];
char *argsep;
unsigned char id;

/*------------------------------------------------
First, copy the command and convert it to all
uppercase.
------------------------------------------------*/
strncpy (cmdstr_buf, cmd, sizeof (cmdstr_buf) - 1);
cmdstr_buf [sizeof (cmdstr_buf) - 1] = '\0';
strupr (cmdstr_buf);

/*------------------------------------------------
Next, find the end of the first thing in the
buffer.  Since the command ends with a space,
we'll look for that.  NULL-Terminate the command
and keep a pointer to the arguments.
------------------------------------------------*/
argsep = strchr (cmdstr_buf, ' ');

if (argsep == NULL)
  {
  argstr_buf [0] = '\0';
  }
else
  {
  strcpy (argstr_buf, argsep + 1);
  *argsep = '\0';
  }

/*------------------------------------------------
Search for a command ID, then switch on it.  Note
that I removed my action items for each command,
but you would invoke each function here.
------------------------------------------------*/
id = cmdid_search (cmdstr_buf);

switch (id)
  {
  case CID_SET_CLK:	break;
  case CID_READ_CLK:	break;
  case CID_SCAN_CLK:    break;
  case CID_SET_ALM:	break;
  case CID_READ_ALM:	break;
  case CID_CLR_ALM:	break;

  case CID_LAST:
CMDERR:
    com_puts (helptext);
    break;
  }
}

